<?php
/**
 *  Author: Martin Tomasek
 *  License: https://creativecommons.org/licenses/by-sa/4.0/ CC BY-SA 4.0
 */

ini_set('max_execution_time', '0');
ini_set('display_errors', 0);
error_reporting(E_ALL);

include_once(dirname(__FILE__).'/../../config/config.inc.php');

function customErrorHandler($errno, $errstr, $errfile, $errline) {
    echo "<html><body>Error: $errstr in $errfile on line $errline.</body></html>";
    http_response_code(500);
    exit;
}

set_error_handler('customErrorHandler');

$feed_name = Tools::getValue('feed');
$key = Tools::getValue("key");
$SAMBA_KEY = Configuration::get('SAMBA_KEY');

if ($key != $SAMBA_KEY) {
    echo "<html><body>Wrong key, access denied.</body></html>";
    http_response_code(403);
    return;
}

$feedFolder = 'feeds/';
$filePath = $feedFolder . $feed_name . '_done.xml';

if (!file_exists($filePath)) {
    echo "<html><body>File not found.</body></html>";
    http_response_code(404);
    return;
}

$type = Tools::getValue('type', 'inline');

header('Content-Type: text/xml');
header('Content-Disposition: ' . ($type == 'download' ? 'attachment' : 'inline') . '; filename="' . basename($filePath) . '"');
header('Content-Length: ' . filesize($filePath));

try {
    $file = fopen($filePath, 'rb');
    if ($file) {
        while (!feof($file)) {
            echo fread($file, 8192);
            ob_flush();
            flush();
        }
        fclose($file);
    } else {
        throw new Exception('Failed to open file.');
    }
} catch (Exception $e) {
    echo "<html><body>Error: " . $e->getMessage() . ".</body></html>";
    http_response_code(500);
}

exit;

