<?php
/**
*  @author    Martin Tomasek
*  @copyright DiffSolutions, s.r.o.
*  @license   https://creativecommons.org/licenses/by-sa/4.0/ CC BY-SA 4.0
*/

include_once('fcommon.php');
include_once('fcategory.php');

function price_with_tax($id_product, $id_product_attribute, $use_reduction)
{
    return Product::getPriceStatic(
        (int) $id_product,
        true,
        (int) $id_product_attribute,
        2,
        null,
        false,
        $use_reduction
    );
}

class FProduct
{
    public function __construct($w, $common)
    {
      $this->w = $w;
      $this->common = $common;
      $ID_LANG = Configuration::get('SAMBA_LANG');
      $ID_SHOP = Configuration::get('SAMBA_SHOP');
      $this->q = "select p.*, i.id_image as image, pl.*, ".
        "cl.link_rewrite as cat_link_rewrite, s.quantity as stock, mfg.name as mfg_desc ".
        'from '.$common->tname('product').' as p '.
        $common->tjoin('product_lang', 'pl').' on pl.id_product = p.id_product '.
        $common->tjoin('image', 'i').' on p.id_product = i.id_product '.
        $common->tjoin('stock_available', 's').' on s.id_product = p.id_product '.
        $common->tjoin('product_shop', 'ps').' on ps.id_product = p.id_product '.
  	    $common->tjoin('category_lang', 'cl').' on cl.id_category = p.id_category_default '.
  	    $common->tjoin('manufacturer', 'mfg').' on mfg.id_manufacturer = p.id_manufacturer '.
  	    "inner join (select id_product,min(position) as position from ".
	    	$common->tname("image").
  	    " group by id_product) img_pos on img_pos.id_product = i.id_product and img_pos.position = i.position ".
            'where pl.id_lang = '.$ID_LANG.' and '.
                'ps.id_shop = '.$ID_SHOP.' and '.
                's.id_product_attribute = 0 and '.
                'cl.id_lang = '.$ID_LANG.' '.
		'group by p.id_product';
    }


    public function genFeedPage($page, $cats)
    {
        $pagesize = 1000;
        $products = Db::getInstance()->ExecuteS($this->q . ' order by p.id_product limit 1000 offset ' . $page * $pagesize);
        $productsVariants = Configuration::get('SAMBA_PRODUCTS_VARIANTS');
        $this->genProducts($products,$productsVariants,$cats);
	return count($products);
    }

    public function genFeed()
    {
        $w = $this->w;
        $common = $this->common;

        $categories = new FCategory($w, $common, false);
        $cats = $categories->genFeed(); # returns indexed categories
        $l = 1;
	$pageno = 0;
	while($l){
		$l = $this->genFeedPage($pageno, $cats);
		$pageno += 1;
	}

        $w->end();
        #return $products;
    }

    public function variants($db, $product_id, $link_rewrite, $image)
    {
        if (empty($product_id)) {
            return false;
        }
        $common = $this->common;
        $q='select pa.*, pac.*, sa.quantity as stock from '.
        $common->tname('product_attribute', 'pa').' '.
        $common->tjoin('product_attribute_combination', 'pac').
	      ' on pa.id_product_attribute = pac.id_product_attribute '.
	      $common->tjoin('stock_available', 'sa').
	      ' on pa.id_product_attribute = sa.id_product_attribute '.
	      'where pa.id_product = '.$product_id.' '.
	      ' and sa.id_product = '.$product_id.' '.
        'order by pac.id_product_attribute';
        $vs = $db->ExecuteS($q);
        $default_ixs = array();
        foreach ($vs as $ix => $v) {
            if ($v['default_on'] == '1') {
                $default_ixs[] = $ix;
            }
        }
        if ($default_ixs) {
            $default_ix = $default_ixs[0];
            if ($default_ix > 0) {
                $tmp = $vs[$default_ix];
                $vs[$default_ix] = $vs[0];
                $vs[0] = $tmp;
            }
	}

	if (! $vs){
		return false;
	}
  foreach ($common->groupby($vs, 'id_product_attribute') as $id => $vg) {
    if (empty($vg)) {
      continue;
    }
    $id_product_attribute = $vg[0]['id_product_attribute'];
    $variant_id = $product_id.'-'.$id_product_attribute;
    $xw = $this->w->getWriter();
    $xw->startElement('VARIANT');
    $xw->writeElement('PRODUCT_ID', $variant_id);
    $xw->writeElement(
    'PRICE',
    price_with_tax((int)$product_id, (int)$id_product_attribute, true)
    );
    if($vg[0]['wholesale_price'] > 0)
      $xw->writeElement(
      'PRICE_BUY',
      $vg[0]['wholesale_price']
      );
    $xw->writeElement(
    'PRICE_BEFORE_DISCOUNT',
    price_with_tax(
    (int)$product_id,
    (int)$id_product_attribute,
    false
    )
    );
    $xw->writeElement('IMAGE', $common->imgUrl(Product::getCover((int)$product_id)['id_image'], $link_rewrite));
    $xw->writeElement('URL',$common->genUrls('variant', $product_id, $id_product_attribute));
    $stock = $vg[0]['stock'];
    if ($stock < 0)
        $stock = 0;
    $xw->writeElement('STOCK', $stock);

    #TODO: parameters
    #$parameters = array();
    $xw->endElement();
  }

  return true;
    }

    public function variantsAsProducts($db, $product_id, $link_rewrite, $image, $productsVariants,$p,$cats)
    {
        if (empty($product_id)) {
            return false;
        }
        $w = $this->w;
        $ID_LANG = Configuration::get('SAMBA_LANG');
        $ID_SHOP = Configuration::get('SAMBA_SHOP');
        $DESCRIPTION = Configuration::get('SAMBA_DESCRIPTION');
        $genGroups = Configuration::get('SAMBA_PRODUCTS_GROUP_PRICE');
        $EMPLOYEE_ID = $this->common->getConfig('EMPLOYEE_ID');
        $sGroups = Group::getGroups($ID_LANG);
        $context = Context::getContext();
        $employee = new Employee($EMPLOYEE_ID, $ID_LANG, $ID_SHOP);
        $context->employee = $employee;
        $common = $this->common;
        $q='select pa.*, pac.*, sa.quantity as stock from '.$common->tname('product_attribute', 'pa').' '.
        $common->tjoin('product_attribute_combination', 'pac').
        ' on pa.id_product_attribute = pac.id_product_attribute '.
        $common->tjoin('stock_available', 'sa').
        ' on pa.id_product_attribute = sa.id_product_attribute '.
        'where pa.id_product = '.$product_id.' '.
        ' and sa.id_product = '.$product_id.' '.
        'order by pac.id_product_attribute';
        $vs = $db->ExecuteS($q);
        $default_ixs = array();
        foreach ($vs as $ix => $v) {
            if ($v['default_on'] == '1') {
                $default_ixs[] = $ix;
            }
        }
        if ($default_ixs) {
            $default_ix = $default_ixs[0];
            if ($default_ix > 0) {
                $tmp = $vs[$default_ix];
                $vs[$default_ix] = $vs[0];
                $vs[0] = $tmp;
            }
  }

  if (! $vs){
    return false;
  }
  foreach ($common->groupby($vs, 'id_product_attribute') as $id => $vg) {
    if (empty($vg)) {
      continue;
    }
    $id_product_attribute = $vg[0]['id_product_attribute'];
    $variant_id = $product_id.'-'.$id_product_attribute;
    $xw = $this->w->getWriter();
    $xw->startElement('PRODUCT');
    $xw->writeElement('PRODUCT_ID', $variant_id);
    $xw->writeElement('TITLE', $p['name']);
    $xw->writeElement('DESCRIPTION', 'BRAK');
    $aGroups = $this->getGroupPrice($sGroups,$product_id,$context,(int)$id_product_attribute);
    if(isset($aGroups) AND $genGroups){
      $xw->startElement('PRICES');
      foreach ($aGroups as $aGroup) {
        $xw->startElement('CATEGORY');
        $xw->writeElement('NAME', $aGroup['name']);
        $xw->writeElement('PRICE', $aGroup['price']);
        $xw->writeElement('PRICE_BEFORE_DISCOUNT', price_with_tax((int)$product_id, (int)$id_product_attribute, false));
        $xw->endElement();
      }
      $xw->endElement();
    }
    $xw->writeElement(
    'PRICE',
    price_with_tax((int)$product_id, (int)$id_product_attribute, true)
    );
    if($vg[0]['wholesale_price'] > 0)
      $xw->writeElement(
      'PRICE_BUY',
      $vg[0]['wholesale_price']
      );
    $xw->writeElement(
    'PRICE_BEFORE_DISCOUNT',
    price_with_tax(
    (int)$product_id,
    (int)$id_product_attribute,
    false
    )
    );
    $frontfeatures = Product::getFrontFeaturesStatic(1, $product_id);
    if(isset($frontfeatures)){

      $xw->startElement('PARAMETERS');
      foreach ($frontfeatures as $feature) {
        $xw->startElement('PARAMETER');
        $xw->writeElement('NAME', $feature['name']);
        $xw->writeElement('VALUE', $feature['value']);
        $xw->endElement();
      }
      $xw->endElement();
    }

    $xw->writeElement('IMAGE', $common->imgUrl(Product::getCover((int)$product_id)['id_image'], $link_rewrite));
    $xw->writeElement('URL',$common->genUrls('variant', $product_id, $id_product_attribute));
    if (array_key_exists('cattext', $cats[$p['id_category_default']])) {
        $cattext = ($cats[$p['id_category_default']])['cattext'];
        if($cattext)
        $xw->writeElement('CATEGORYTEXT', $cattext);
    }
    $stock = $vg[0]['stock'];
    if ($stock < 0)
        $stock = 0;

    $xw->writeElement('STOCK', $stock);

    #TODO: parameters
    #$parameters = array();
    $xw->endElement();
  }

  return true;
    }


    public function getGroupPrice($groups,$id_product,$context,$id_product_attribute = null){
    $aGroup = array();
    $ID_SHOP = Configuration::get('SAMBA_SHOP');
    $ID_CURRENCY = Currency::getDefaultCurrency()->id;
    $ID_COUNTRY = $context->country->id;
    foreach ($groups as $group) {
        $reduction_value = 0;
        $price = Product::getPriceStatic($id_product,true,$id_product_attribute,2,null,false,null);
        $today = date('y-m-d');
        $reduction = '';
        $spec_reductions = Db::getInstance()->executeS("SELECT * FROM " . _DB_PREFIX_ . "specific_price
        WHERE
        id_shop = '$ID_SHOP' AND
        id_product = $id_product AND
        id_product_attribute = '" . $id_product_attribute . "'
        ");
        array_multisort(
            array_map(
                static function ($element) {
                    return $element['reduction'];
                },
                $spec_reductions
            ),
            SORT_DESC,
            $spec_reductions
        );
        $id_group = $group['id_group'];
        if  ($group["reduction"] > 0){
            if ($reduction < $group['reduction']) {
                $reduction = ($group['reduction']/100);
                $reduction_value = $price*($reduction);
                $aGroup[$id_group]['reduction'] = $reduction;
                $aGroup[$id_group]['name'] = $group['name'];
                $aGroup[$id_group]['reduction_value'] = $reduction_value;
                $aGroup[$id_group]['price'] = round($price - $reduction_value,2);
            }
        } else {
            $aGroup[$id_group]['reduction'] = '';
            $aGroup[$id_group]['name'] = $group['name'];
            $aGroup[$id_group]['reduction_value'] = '';
            $aGroup[$id_group]['price'] = round($price - $reduction_value,2);
        }
        foreach ($spec_reductions as $spec_reduction) {
            if ((strtotime($spec_reduction['to']) > time() && strtotime($spec_reduction['from']) < time()) || $spec_reduction['to'] == '0000-00-00 00:00:00') {
                if ($spec_reduction['id_group'] == $id_group || $spec_reduction['id_group']==0) {
                    if( $spec_reduction['price'] > 0){
                        $aGroup[$id_group]['price'] = round($spec_reduction['price'] * (1 + (Tax::getProductTaxRate($id_product) / 100)),2);
                    } elseif ($spec_reduction['reduction_type'] == 'percentage'){
                        $reduction_cache =  $price*$spec_reduction['reduction'];
                        if ($reduction_cache > $reduction_value) {
                            $reduction = $spec_reduction['reduction'];
                            $reduction_value = $reduction_cache;
                            $aGroup[$id_group]['reduction'] = $reduction;
                            $aGroup[$id_group]['name'] = $group['name'];
                            $aGroup[$id_group]['reduction_value'] = $reduction_value;
                            $aGroup[$id_group]['price'] = round($price - $reduction_value,2);
                        }
                    } elseif ($spec_reduction['reduction_type'] == 'amount'){
                        if($spec_reduction['reduction'] > $reduction_value){
                            $reduction_value = $spec_reduction['reduction'];
                            if ($spec_reduction['reduction_tax'] == 0 )
                              $reduction_value *=  (1+ (Tax::getProductTaxRate($id_product)/100));
                            $aGroup[$id_group]['reduction'] = $reduction;
                            $aGroup[$id_group]['name'] = $group['name'];
                            $aGroup[$id_group]['reduction_value'] = $reduction_value;
                            $aGroup[$id_group]['price'] = round($price - $reduction_value,2);
                        }
                    }
                }
            }
        }
    }
    return $aGroup;
  }

    public function genProducts($products,$productsVariants,$cats){
      $db = Db::getInstance();
      if ($productsVariants) {
        foreach ($products as $p) {
            $product_id = $p['id_product'];
            $product = new Product((int)$product_id);
            if (!$product->hasAttributes()) {
              $this->genProduct($p,$productsVariants,$cats,$db);
            }
            if ($this->variantsAsProducts($db, $product_id, $p['link_rewrite'], Product::getCover((int)$product_id)['id_image'],$productsVariants,$p,$cats)){
            }
        }
      } else {
        foreach ($products as $p) {
          $this->genProduct($p,$productsVariants,$cats,$db);
        }
      }
    }

    public function genProduct($p,$productsVariants,$cats,$db){
      $w = $this->w;
      $common = $this->common;
      $ID_LANG = Configuration::get('SAMBA_LANG');
      $ID_SHOP = Configuration::get('SAMBA_SHOP');
      $genGroups = Configuration::get('SAMBA_PRODUCTS_GROUP_PRICE');
      $DESCRIPTION = Configuration::get('SAMBA_DESCRIPTION');
      $EMPLOYEE_ID = $this->common->getConfig('EMPLOYEE_ID');
      $sGroups = Group::getGroups($ID_LANG);
      $context = Context::getContext();
      $employee = new Employee($EMPLOYEE_ID, $ID_LANG, $ID_SHOP);
      $context->employee = $employee;
      $xw = $w->startLn();
      $product_id = $p['id_product'];
      $xw->writeElement('PRODUCT_ID', $product_id.'-0');
      $xw->writeElement('TITLE', $p['name']);
      $xw->writeElement('DESCRIPTION', 'BRAK');
      $xw->writeElement('URL',$common->genUrls('product',$product_id));
      $xw->writeElement('IMAGE', $common->imgUrl(Product::getCover((int)$product_id)['id_image'], $p['link_rewrite']));
      if (array_key_exists('cattext', $cats[$p['id_category_default']])) {
          $cattext = ($cats[$p['id_category_default']])['cattext'];
          if ($cattext)
          $xw->writeElement('CATEGORYTEXT', $cattext);
      }
  //Adding Product Features
  $aGroups = $this->getGroupPrice($sGroups,$product_id,$context);
  if(isset($aGroups) AND $genGroups){
    $xw->startElement('PRICES');
    foreach ($aGroups as $aGroup) {
      $xw->startElement('CATEGORY');
      $xw->writeElement('NAME', $aGroup['name']);
      $xw->writeElement('PRICE', $aGroup['price']);
      $xw->writeElement('PRICE_BEFORE_DISCOUNT', price_with_tax((int)$product_id, null, false));
      $xw->endElement();
    }
    $xw->endElement();
  }
  $frontfeatures = Product::getFrontFeaturesStatic(1, $product_id);
  if(isset($frontfeatures)){

    $xw->startElement('PARAMETERS');
    foreach ($frontfeatures as $feature) {
      $xw->startElement('PARAMETER');
      $xw->writeElement('NAME', $feature['name']);
      $xw->writeElement('VALUE', $feature['value']);
      $xw->endElement();
    }
    $xw->endElement();
  }

      $stock = $p['stock'];
      if (!$p['active'])
          $stock = 0;

      if ($p['visibility'] == 'none')
          $stock = 0;

      if ($p['show_price'] <= 0)
          $stock = 0;

      if ($stock < 0)
          $stock = 0;
      $xw->writeElement('PRICE', price_with_tax((int)$product_id, null, true));
      $xw->writeElement('PRICE_BEFORE_DISCOUNT', price_with_tax((int)$product_id, null, false));

      $wsp = $p['wholesale_price'];
      if ($wsp && $wsp>0) {
          $xw->writeElement('PRICE_BUY', (int)((float)$wsp *100)/100);
      } #buy price rouded to 2 decimal places

      $xw->writeElement('BRAND', $p['mfg_desc']);
      if ($productsVariants) {
        $xw->writeElement('STOCK', $stock);
      } else {
        if ($this->variants($db, $product_id, $p['link_rewrite'], Product::getCover((int)$product_id)['id_image'])){
            $xw->writeElement('STOCK', 0);
      }
      }
      $w->endLn();
    }
}
